@@ -71,6 +71,7 @@ impl InputHandler {
                         Status::Error,
                         Status::Help,
                         Status::DeleteConfirm,
+                        Status::Filter,
                     ]) {
                         self.mouse_press(mouse_event);
                     }
@@ -125,7 +126,7 @@ impl InputHandler {
         let is_oxker = self.app_data.lock().is_oxker();
         if !is_oxker && tty_readable() {
             let uuid = Uuid::new_v4();
-            let handle = GuiState::start_loading_animation(&self.gui_state, uuid);
+            GuiState::start_loading_animation(&self.gui_state, uuid);
             let (sx, rx) = tokio::sync::oneshot::channel::<Arc<Docker>>();
             self.docker_tx.send(DockerMessage::Exec(sx)).await.ok();
 
@@ -143,7 +144,7 @@ impl InputHandler {
                     },
                 );
             }
-            self.gui_state.lock().stop_loading_animation(&handle, uuid);
+            self.gui_state.lock().stop_loading_animation(uuid);
         }
     }
 
@@ -177,7 +178,7 @@ impl InputHandler {
     }
 
     /// Save the currently selected containers logs into a `[container_name]_[timestamp].log` file
-    async fn s_key(&mut self) {
+    async fn s_key(&self) {
         /// This is the inner workings, *inlined* here to return a Result
         async fn save_logs(
             app_data: &Arc<Mutex<AppData>>,
@@ -248,7 +249,7 @@ impl InputHandler {
             self.gui_state.lock().status_push(log_status);
 
             let uuid = Uuid::new_v4();
-            let handle = GuiState::start_loading_animation(&self.gui_state, uuid);
+            GuiState::start_loading_animation(&self.gui_state, uuid);
             if save_logs(&self.app_data, &self.gui_state, &self.docker_tx)
                 .await
                 .is_err()
@@ -260,12 +261,12 @@ impl InputHandler {
                 );
             }
             self.gui_state.lock().status_del(log_status);
-            self.gui_state.lock().stop_loading_animation(&handle, uuid);
+            self.gui_state.lock().stop_loading_animation(uuid);
         }
     }
 
     /// Send docker command, if the Commands panel is selected
-    async fn enter_key(&mut self) {
+    async fn enter_key(&self) {
         // This isn't great, just means you can't send docker commands before full initialization of the program
         let panel = self.gui_state.lock().get_selected_panel();
         if panel == SelectablePanel::Commands {
@@ -306,7 +307,7 @@ impl InputHandler {
     }
 
     /// Change the the "next" selectable panel
-    fn tab_key(&mut self) {
+    fn tab_key(&self) {
         let is_containers =
             self.gui_state.lock().get_selected_panel() == SelectablePanel::Containers;
         let count = if self.app_data.lock().get_container_len() == 0 && is_containers {
@@ -320,7 +321,7 @@ impl InputHandler {
     }
 
     /// Change to previously selected panel
-    fn back_tab_key(&mut self) {
+    fn back_tab_key(&self) {
         let is_containers = self.gui_state.lock().get_selected_panel() == SelectablePanel::Logs;
         let count = if self.app_data.lock().get_container_len() == 0 && is_containers {
             2
@@ -332,7 +333,7 @@ impl InputHandler {
         }
     }
 
-    fn home_key(&mut self) {
+    fn home_key(&self) {
         let mut locked_data = self.app_data.lock();
         let selected_panel = self.gui_state.lock().get_selected_panel();
         match selected_panel {
@@ -343,7 +344,7 @@ impl InputHandler {
     }
 
     /// Go to end of the list of the currently selected panel
-    fn end_key(&mut self) {
+    fn end_key(&self) {
         let mut locked_data = self.app_data.lock();
         let selected_panel = self.gui_state.lock().get_selected_panel();
         match selected_panel {
@@ -353,6 +354,104 @@ impl InputHandler {
         }
     }
 
+    /// Actions to take when in Help status active
+    fn handle_help(&mut self, key_code: KeyCode) {
+        match key_code {
+            KeyCode::Esc | KeyCode::Char('h' | 'H') => {
+                self.gui_state.lock().status_del(Status::Help);
+            }
+            KeyCode::Char('m' | 'M') => self.m_key(),
+            _ => (),
+        }
+    }
+
+    /// Actions to take when Error status active
+    fn handle_error(&self, key_code: KeyCode) {
+        match key_code {
+            KeyCode::Esc | KeyCode::Char('c' | 'C') => {
+                self.app_data.lock().remove_error();
+                self.gui_state.lock().status_del(Status::Error);
+            }
+            _ => (),
+        }
+    }
+
+    /// Actions to take when Delete status active
+    async fn handle_delete(&self, key_code: KeyCode) {
+        match key_code {
+            KeyCode::Char('y' | 'Y') => self.confirm_delete().await,
+            KeyCode::Esc | KeyCode::Char('n' | 'N') => self.clear_delete(),
+            _ => (),
+        }
+    }
+
+    /// Actions to take when Filter status active
+    fn handle_filter(&self, key_code: KeyCode) {
+        match key_code {
+            KeyCode::Esc => {
+                self.app_data.lock().filter_term_clear();
+                self.gui_state.lock().status_del(Status::Filter);
+            }
+            KeyCode::Enter | KeyCode::F(1) | KeyCode::Char('/') => {
+                self.gui_state.lock().status_del(Status::Filter);
+            }
+            KeyCode::Backspace => {
+                self.app_data.lock().filter_term_pop();
+            }
+            KeyCode::Char(x) => {
+                self.app_data.lock().filter_term_push(x);
+            }
+            KeyCode::Right => {
+                self.app_data.lock().filter_by_next();
+            }
+            KeyCode::Left => {
+                self.app_data.lock().filter_by_prev();
+            }
+            _ => (),
+        }
+    }
+
+    /// Handle button presses in all other scenarios
+    async fn handle_others(&mut self, key_code: KeyCode) {
+        match key_code {
+            KeyCode::Char('0') => self.app_data.lock().reset_sorted(),
+            KeyCode::Char('1') => self.sort(Header::Name),
+            KeyCode::Char('2') => self.sort(Header::State),
+            KeyCode::Char('3') => self.sort(Header::Status),
+            KeyCode::Char('4') => self.sort(Header::Cpu),
+            KeyCode::Char('5') => self.sort(Header::Memory),
+            KeyCode::Char('6') => self.sort(Header::Id),
+            KeyCode::Char('7') => self.sort(Header::Image),
+            KeyCode::Char('8') => self.sort(Header::Rx),
+            KeyCode::Char('9') => self.sort(Header::Tx),
+            KeyCode::Char('e' | 'E') => self.e_key().await,
+            KeyCode::Char('h' | 'H') => self.gui_state.lock().status_push(Status::Help),
+            KeyCode::Char('m' | 'M') => self.m_key(),
+            KeyCode::Char('s' | 'S') => self.s_key().await,
+            KeyCode::Tab => self.tab_key(),
+            KeyCode::BackTab => self.back_tab_key(),
+            KeyCode::Home => self.home_key(),
+            KeyCode::End => self.end_key(),
+            KeyCode::Up | KeyCode::Char('k' | 'K') => self.previous(),
+            KeyCode::PageUp => {
+                for _ in 0..=6 {
+                    self.previous();
+                }
+            }
+            KeyCode::F(1) | KeyCode::Char('/') => {
+                self.gui_state.lock().status_push(Status::Filter);
+                self.docker_tx.send(DockerMessage::Update).await.ok();
+            }
+            KeyCode::Down | KeyCode::Char('j' | 'J') => self.next(),
+            KeyCode::PageDown => {
+                for _ in 0..=6 {
+                    self.next();
+                }
+            }
+            KeyCode::Enter => self.enter_key().await,
+            _ => (),
+        }
+    }
     /// Handle keyboard button events
     async fn button_press(&mut self, key_code: KeyCode, key_modifier: KeyModifiers) {
         let contains_delete = self
@@ -365,78 +464,32 @@ impl InputHandler {
         let contains_error = contains(Status::Error);
         let contains_help = contains(Status::Help);
         let contains_exec = contains(Status::Exec);
+        let contains_filter: bool = contains(Status::Filter);
 
         if !contains_exec {
-            // Always just quit on Ctrl + c/C or q/Q
             let is_c = || key_code == KeyCode::Char('c') || key_code == KeyCode::Char('C');
             let is_q = || key_code == KeyCode::Char('q') || key_code == KeyCode::Char('Q');
-            if key_modifier == KeyModifiers::CONTROL && is_c() || is_q() {
+            if key_modifier == KeyModifiers::CONTROL && is_c() || is_q() && !contains_filter {
+                // Always just quit on Ctrl + c/C or q/Q, unless in FIlter status active
                 self.quit().await;
             }
 
             if contains_error {
-                match key_code {
-                    KeyCode::Esc | KeyCode::Char('c' | 'C') => {
-                        self.app_data.lock().remove_error();
-                        self.gui_state.lock().status_del(Status::Error);
-                    }
-                    _ => (),
-                }
+                self.handle_error(key_code);
             } else if contains_help {
-                match key_code {
-                    KeyCode::Esc | KeyCode::Char('h' | 'H') => {
-                        self.gui_state.lock().status_del(Status::Help);
-                    }
-                    KeyCode::Char('m' | 'M') => self.m_key(),
-                    _ => (),
-                }
+                self.handle_help(key_code);
+            } else if contains_filter {
+                self.handle_filter(key_code);
             } else if contains_delete {
-                match key_code {
-                    KeyCode::Char('y' | 'Y') => self.confirm_delete().await,
-                    KeyCode::Esc | KeyCode::Char('n' | 'N') => self.clear_delete(),
-                    _ => (),
-                }
+                self.handle_delete(key_code).await;
             } else {
-                match key_code {
-                    KeyCode::Char('0') => self.app_data.lock().reset_sorted(),
-                    KeyCode::Char('1') => self.sort(Header::Name),
-                    KeyCode::Char('2') => self.sort(Header::State),
-                    KeyCode::Char('3') => self.sort(Header::Status),
-                    KeyCode::Char('4') => self.sort(Header::Cpu),
-                    KeyCode::Char('5') => self.sort(Header::Memory),
-                    KeyCode::Char('6') => self.sort(Header::Id),
-                    KeyCode::Char('7') => self.sort(Header::Image),
-                    KeyCode::Char('8') => self.sort(Header::Rx),
-                    KeyCode::Char('9') => self.sort(Header::Tx),
-                    KeyCode::Char('e' | 'E') => self.e_key().await,
-                    KeyCode::Char('h' | 'H') => self.gui_state.lock().status_push(Status::Help),
-                    KeyCode::Char('m' | 'M') => self.m_key(),
-                    KeyCode::Char('s' | 'S') => self.s_key().await,
-                    KeyCode::Tab => self.tab_key(),
-                    KeyCode::BackTab => self.back_tab_key(),
-                    KeyCode::Home => self.home_key(),
-                    KeyCode::End => self.end_key(),
-                    KeyCode::Up | KeyCode::Char('k' | 'K') => self.previous(),
-                    KeyCode::PageUp => {
-                        for _ in 0..=6 {
-                            self.previous();
-                        }
-                    }
-                    KeyCode::Down | KeyCode::Char('j' | 'J') => self.next(),
-                    KeyCode::PageDown => {
-                        for _ in 0..=6 {
-                            self.next();
-                        }
-                    }
-                    KeyCode::Enter => self.enter_key().await,
-                    _ => (),
-                }
+                self.handle_others(key_code).await;
             }
         }
     }
 
     /// Check if a button press interacts with either the yes or no buttons in the delete container confirm window
-    async fn button_intersect(&mut self, mouse_event: MouseEvent) {
+    async fn button_intersect(&self, mouse_event: MouseEvent) {
         if mouse_event.kind == MouseEventKind::Down(MouseButton::Left) {
             let intersect = self.gui_state.lock().button_intersect(Rect::new(
                 mouse_event.column,
@@ -455,17 +508,18 @@ impl InputHandler {
     }
 
     /// Handle mouse button events
-    fn mouse_press(&mut self, mouse_event: MouseEvent) {
+    fn mouse_press(&self, mouse_event: MouseEvent) {
         match mouse_event.kind {
             MouseEventKind::ScrollUp => self.previous(),
             MouseEventKind::ScrollDown => self.next(),
             MouseEventKind::Down(MouseButton::Left) => {
-                if let Some(header) = self.gui_state.lock().header_intersect(Rect::new(
+                let header = self.gui_state.lock().header_intersect(Rect::new(
                     mouse_event.column,
                     mouse_event.row,
                     1,
                     1,
-                )) {
+                ));
+                if let Some(header) = header {
                     self.sort(header);
                 }
 
@@ -481,7 +535,7 @@ impl InputHandler {
     }
 
     /// Change state to next, depending which panel is currently in focus
-    fn next(&mut self) {
+    fn next(&self) {
         let mut locked_data = self.app_data.lock();
         let selected_panel = self.gui_state.lock().get_selected_panel();
         match selected_panel {
@@ -492,7 +546,7 @@ impl InputHandler {
     }
 
     /// Change state to previous, depending which panel is currently in focus
-    fn previous(&mut self) {
+    fn previous(&self) {
         let mut locked_data = self.app_data.lock();
         let selected_panel = self.gui_state.lock().get_selected_panel();
         match selected_panel {
